#!/bin/sh
#
# Copyright (c) 2010  Peter Pentchev
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.

set -e

basedir='/sys/class/power_supply'
pattern='BAT'

if [ ! -d "$basedir" ]; then
	echo "No power supply base: $basedir" 1>&2
	exit 1
fi
cd "$basedir"
sub=`ls -d "$pattern"*`
if [ -z "$sub" ]; then
	echo "No batteries" 1>&2
	exit 1
fi

echo "Processing batteries: $sub"
for dev in $sub; do
	echo "Device $dev"
	fnow="$dev/charge_now"
	ffull="$dev/charge_full"
	if [ ! -r "$fnow" ] || [ ! -r "$ffull" ]; then
		echo "Unreadable $fnow or $ffull" 1>&2
		continue;
	fi
	cnow=`cat "$fnow"`
	cfull=`cat "$ffull"`
	if [ -z "$cnow" ]; then
		echo "Could not read the current charge from $fnow" 1>&2
		continue
	fi
	if ! expr "x$cnow" : 'x[1-9][0-9]*$' > /dev/null; then
		echo "Not a number in the current charge file $fnow" 1>&2 1>&2
		continue
	fi
	if [ -z "$cfull" ]; then
		echo "Could not read the full charge from $ffull" 1>&2
		continue
	fi
	if ! expr "x$cfull" : 'x[1-9][0-9]*$' > /dev/null; then
		echo "Not a number in the full charge file $ffull" 1>&2 1>&2
		continue
	fi
	if [ "$cfull" = 0 ]; then
		echo "Weirdness, zero full charge in $ffull" 1>&2
		continue
	fi

	printf '%s\t%d\t%d\t%.2f\n' "$dev" "$cnow" "$cfull" `expr "$cnow" \* 100 / "$cfull"`
done
