#!/bin/sh
#
# Copyright (c) 2010  Peter Pentchev
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.

set -e

header='1'
opts=''
while getopts 'husfVo' o; do
	if [ "$o" = '?' ]; then
		w '-?' 2>&1 | sed -ne '1!p' 1>&2
		exit 1
	fi
	if [ "$o" = 'h' ]; then
		header=''
	fi
	opts="$opts$o"
done
shift `expr "$OPTIND" - 1`

tempfile=`mktemp -t ws.XXXXXX`
if [ -z "$tempfile" ] || [ ! -f "$tempfile" ]; then
	echo 'Could not create a temporary file' 1>&2
	exit 1
fi
trap "rm -f -- \"$tempfile\"" EXIT HUP INT QUIT TERM
if [ ! -r "$tempfile" ] || [ ! -w "$tempfile" ]; then
	echo 'Unusable temporary file' 1>&2
	exit 1
fi

if [ -z "$COLUMNS" ]; then
	# Okay then, we assume we are running on a GNU/Linux system (else why
	# would there be any need for a 'sorted w'? :) BSD systems do it
	# by default :) ...so we assume a GNU-style stty with "columns 105"
	# instead of "105 columns".  It's trivial to add a second s/// to
	# the sed(1) invocation, and I will do that if anybody requests it.
	#
	cols=`stty -a | sed -e 's/.*columns \([1-9][0-9]*\).*/\1/; q'`
	if [ -n "$cols" ]; then
		COLUMNS="$cols"
		export COLUMNS
	fi
fi

w ${opts:+"-$opts"} | while read line; do
	if [ -n "$header" ]; then
		echo "$line"
		if expr "x$line" : '^xUSER' > /dev/null; then
			header=''
		fi
	else
		echo "$line" >> "$tempfile"
	fi
done

# Bah, it would be nice to have some combination of alpha- and numeric
# sorting, so that tty1 and tty10 are not lumped together before tty2.
sort -k2,2 -- "$tempfile"
